@@ -1,17 +1,14 @@
 """Pickle field implementation for Django."""
-
 from copy import deepcopy
 from base64 import b64encode, b64decode
 from zlib import compress, decompress
-try:
-    from cPickle import loads, dumps
-except ImportError:
-    from pickle import loads, dumps
-
+import six
+import django
 from django.db import models
-from django.utils.encoding import force_unicode
 
 from picklefield import DEFAULT_PROTOCOL
+from picklefield.compat import force_text, loads, dumps
+
 
 class PickledObject(str):
     """
@@ -24,9 +21,9 @@ class PickledObject(str):
     remove PickledObject and its references, you won't be able to pass
     in pre-encoded values anymore, but you can always just pass in the
     python objects themselves.
-
     """
 
+
 class _ObjectWrapper(object):
     """
     A class used to wrap object that have properties that may clash with the 
@@ -41,11 +38,13 @@ class _ObjectWrapper(object):
     def __init__(self, obj):
         self._obj = obj
 
+
 def wrap_conflictual_object(obj):
     if hasattr(obj, 'prepare_database_save') or callable(obj):
         obj = _ObjectWrapper(obj)
     return obj
 
+
 def dbsafe_encode(value, compress_object=False, pickle_protocol=DEFAULT_PROTOCOL):
     # We use deepcopy() here to avoid a problem with cPickle, where dumps
     # can generate different character streams for same lookup value if
@@ -53,22 +52,31 @@ def dbsafe_encode(value, compress_object=False, pickle_protocol=DEFAULT_PROTOCOL
     # The reason this is important is because we do all of our lookups as
     # simple string matches, thus the character streams must be the same
     # for the lookups to work properly. See tests.py for more information.
-    if not compress_object:
-        value = b64encode(dumps(deepcopy(value), pickle_protocol))
-    else:
-        value = b64encode(compress(dumps(deepcopy(value), pickle_protocol)))
+    value = dumps(deepcopy(value), protocol=pickle_protocol)
+    if compress_object:
+        value = compress(value)
+    value = b64encode(value).decode() # decode bytes to str
     return PickledObject(value)
 
 
 def dbsafe_decode(value, compress_object=False):
-    if not compress_object:
-        value = loads(b64decode(value))
-    else:
-        value = loads(decompress(b64decode(value)))
-    return value
+    value = value.encode() # encode str to bytes
+    value = b64decode(value)
+    if compress_object:
+        value = decompress(value)
+    return loads(value)
 
 
-class PickledObjectField(models.Field):
+def _get_subfield_superclass():
+    # hardcore trick to support django < 1.3 - there was something wrong with
+    # inheritance and SubfieldBase before django 1.3
+    # see https://github.com/django/django/commit/222c73261650201f5ce99e8dd4b1ce0d30a69eb4
+    if django.VERSION < (1,3):
+        return models.Field
+    return six.with_metaclass(models.SubfieldBase, models.Field)
+
+
+class PickledObjectField(_get_subfield_superclass()):
     """
     A field that will accept *any* python object and store it in the
     database. PickledObjectField will optionally compress its values if
@@ -78,8 +86,7 @@ class PickledObjectField(models.Field):
     can still do lookups using None). This way, it is still possible to
     use the ``isnull`` lookup type correctly.
     """
-
-    __metaclass__ = models.SubfieldBase
+    __metaclass__ = models.SubfieldBase  # for django < 1.3
 
     def __init__(self, *args, **kwargs):
         self.compress = kwargs.pop('compress', False)
@@ -111,7 +118,7 @@ class PickledObjectField(models.Field):
         B64decode and unpickle the object, optionally decompressing it.
 
         If an error is raised in de-pickling and we're sure the value is
-        a definite pickle, the error is allowed to propogate. If we
+        a definite pickle, the error is allowed to propagate. If we
         aren't sure if the value is a pickle or not, then we catch the
         error and return the original value instead.
 
@@ -145,13 +152,13 @@ class PickledObjectField(models.Field):
 
         """
         if value is not None and not isinstance(value, PickledObject):
-            # We call force_unicode here explicitly, so that the encoded string
+            # We call force_text here explicitly, so that the encoded string
             # isn't rejected by the postgresql_psycopg2 backend. Alternatively,
             # we could have just registered PickledObject with the psycopg
             # marshaller (telling it to store it like it would a string), but
             # since both of these methods result in the same value being stored,
             # doing things this way is much easier.
-            value = force_unicode(dbsafe_encode(value, self.compress, self.protocol))
+            value = force_text(dbsafe_encode(value, self.compress, self.protocol))
         return value
 
     def value_to_string(self, obj):
